@@ -5,13 +5,10 @@ Available detectors: Tristan 10M, Eiger 2X 4M.
 
 import sys
 
-# import json
 import h5py
 import glob
 import logging
 
-import numpy as np
-
 from pathlib import Path
 
 from collections import namedtuple
@@ -32,23 +29,29 @@ from .. import (
 )
 
 from ..nxs_write import (
-    calculate_scan_range,
+    calculate_rotation_scan_range,
 )
 
 from ..nxs_write.NexusWriter import call_writers
-from ..nxs_write.NXclassWriters import write_NXentry
+from ..nxs_write.NXclassWriters import write_NXdatetime, write_NXentry
 
 from ..tools.ExtendedRequest import ExtendedRequestIO
+from ..tools.GDAjson2params import (
+    read_geometry_from_json,
+    read_detector_params_from_json,
+)
 
 # Define a logger object and a formatter
 logger = logging.getLogger("NeXusGenerator.I19-2")
 logger.setLevel(logging.DEBUG)
 formatter = logging.Formatter("%(name)s %(levelname)s %(message)s")  # %(asctime)s
+# Define a stream handler
 CH = logging.StreamHandler(sys.stdout)
 CH.setLevel(logging.DEBUG)
 CH.setFormatter(formatter)
 logger.addHandler(CH)
 
+# Tristan mask and flatfield files
 maskfile = "Tristan10M_mask_with_spec.h5"
 flatfieldfile = "Tristan10M_flat_field_coeff_with_Mo_17.479keV.h5"
 
@@ -60,16 +63,16 @@ tr_collect = namedtuple(
         "detector_name",
         "exposure_time",
         "wavelength",
-        "beam_center",  # This will have a command line call anyway because I can't call it from bash
-        # "beam_pos_x",
-        # "beam_pos_y",
+        "beam_center",
         "start_time",
         "stop_time",
-        "geometry_json",  # Define these 2 ase None
+        "geometry_json",  # Define these 2 as None
         "detector_json",
     ],
 )
 
+coordinate_frame = "mcstas"
+
 # Initialize dictionaries
 goniometer = {}  # goniometer_axes
 detector = {}  # tristan10M_params
@@ -146,20 +149,24 @@ def tristan_writer(
     detector["flatfield"] = flatfieldfile
     # If these two could instead be passed, I'd be happier...
 
+    # Define SCANS dictionary
+    SCANS = {}
+    SCANS["rotation"] = {scan_axis: scan_range}
+
     # Get on with the writing now...
     try:
         with h5py.File(master_file, "x") as nxsfile:
-            nxentry = write_NXentry(nxsfile)
+            write_NXentry(nxsfile)
 
             if timestamps[0]:
-                nxentry.create_dataset("start_time", data=np.string_(timestamps[0]))
+                write_NXdatetime(nxsfile, (timestamps[0], None))
+            #    nxentry.create_dataset("start_time", data=np.string_(timestamps[0]))
 
             call_writers(
                 nxsfile,
                 [TR.meta_file],
-                "mcstas",
-                scan_axis,  # This should be omega
-                scan_range,
+                coordinate_frame,
+                SCANS,
                 (detector["mode"], None),
                 goniometer,
                 detector,
@@ -169,8 +176,10 @@ def tristan_writer(
                 attenuator,
             )
 
+            # write_NXdatetime(nxsfile, (None, timestamps[1]))
             if timestamps[1]:
-                nxentry.create_dataset("end_time", data=np.string_(timestamps[1]))
+                write_NXdatetime(nxsfile, (None, timestamps[1]))
+            #    nxentry.create_dataset("end_time", data=np.string_(timestamps[1]))
             logger.info(f"{master_file} correctly written.")
     except Exception as err:
         logger.exception(err)
@@ -209,27 +218,30 @@ def eiger_writer(
     # Get scan range array
     logger.info("Calculating scan range...")
     scan_idx = goniometer["axes"].index(scan_axis)
-    scan_range = calculate_scan_range(
+    scan_range = calculate_rotation_scan_range(
         goniometer["starts"][scan_idx],
         goniometer["ends"][scan_idx],
         goniometer["increments"][scan_idx],
         n_images=n_frames,
     )
 
+    # Define SCANS dictionary
+    SCANS = {}
+    SCANS["rotation"] = {scan_axis: scan_range}
+
     # Get on with the writing now...
     try:
         with h5py.File(master_file, "x") as nxsfile:
-            nxentry = write_NXentry(nxsfile)
+            write_NXentry(nxsfile)
 
             if timestamps[0]:
-                nxentry.create_dataset("start_time", data=np.string_(timestamps[0]))
+                write_NXdatetime(nxsfile, (timestamps[0], None))
 
             call_writers(
                 nxsfile,
                 filenames,
-                "mcstas",
-                scan_axis,  # This should be omega
-                scan_range,
+                coordinate_frame,
+                SCANS,
                 (detector["mode"], n_frames),
                 goniometer,
                 detector,
@@ -242,7 +254,7 @@ def eiger_writer(
             )
 
             if timestamps[1]:
-                nxentry.create_dataset("end_time", data=np.string_(timestamps[1]))
+                write_NXdatetime(nxsfile, (None, timestamps[1]))
             logger.info(f"{master_file} correctly written.")
     except Exception as err:
         logger.exception(err)
@@ -279,6 +291,13 @@ def write_nxs(**tr_params):
         else None,
     )
 
+    # Define a file handler
+    logfile = TR.meta_file.parent / "nexus_writer.log"
+    FH = logging.FileHandler(logfile, mode="a")
+    FH.setLevel(logging.DEBUG)
+    FH.setFormatter(formatter)
+    logger.addHandler(FH)
+
     logger.info(f"{TR.detector_name} NeXus file writer.")
     logger.info(f"Current collection directory: {TR.meta_file.parent}")
     # Add some information to logger
@@ -288,18 +307,26 @@ def write_nxs(**tr_params):
     logger.info("NeXus file will be saved as %s" % master_file)
 
     # Get goniometer and detector parameters
-    # FIXME I mean, it works but ... TODO
+    # FIXME I mean, it works but ...
     if TR.geometry_json:
-        # here call json reader
-        pass
+        logger.info("Reading geometry from json file.")
+        _gonio, _det = read_geometry_from_json(TR.geometry_json)
+        for k, v in _gonio.items():
+            goniometer[k] = v
+        for k, v in _det.items():
+            detector[k] = v
     else:
+        logger.info("Load goniometer from I19-2.")
         for k, v in goniometer_axes.items():
             goniometer[k] = v
 
     if TR.detector_json:
-        # idem aedem idem
-        pass
+        logger.info("Reading detector parameters from json file.")
+        _det = read_detector_params_from_json(TR.detector_json)
+        for k, v in _det.items():
+            detector[k] = v
     else:
+        logger.info("Load detector parameters for I19-2.")
         if "tristan" in TR.detector_name.lower():
             for k, v in tristan10M_params.items():
                 detector[k] = v
@@ -308,7 +335,8 @@ def write_nxs(**tr_params):
                 detector[k] = v
 
     # Read information from xml file
-    scan_axis, pos, n_frames = read_from_xml(TR.xml_file, TR.detector_name)
+    logger.info("Read xml file.")
+    osc_axis, pos, n_frames = read_from_xml(TR.xml_file, TR.detector_name)
     # n_Frames is only useful for eiger
     # pos[scan_axis][::-1] is scan range
 
@@ -345,7 +373,7 @@ def write_nxs(**tr_params):
     logger.info(f"Timestamps recorded: {timestamps}")
 
     logger.info("Goniometer information")
-    logger.info(f"Scan axis is: {scan_axis}")
+    logger.info(f"Scan axis is: {osc_axis}")
     for j in range(len(goniometer["axes"])):
         logger.info(
             f"Goniometer axis: {goniometer['axes'][j]} => {goniometer['starts'][j]}, {goniometer['types'][j]} on {goniometer['depends'][j]}"
@@ -366,9 +394,9 @@ def write_nxs(**tr_params):
     logger.info(f"Recorded beam center is: {TR.beam_center}.")
 
     if "tristan" in TR.detector_name:
-        tristan_writer(master_file, TR, scan_axis, pos[scan_axis][:-1], timestamps)
+        tristan_writer(master_file, TR, osc_axis, pos[osc_axis][:-1], timestamps)
     else:
-        eiger_writer(master_file, TR, scan_axis, n_frames, timestamps)
+        eiger_writer(master_file, TR, osc_axis, n_frames, timestamps)
 
 
 def main():
