@@ -2,17 +2,16 @@
 Writer functions for different groups of a NeXus file.
 """
 
-import sys
 import h5py
 import logging
 
 import numpy as np
 
 from pathlib import Path
+from datetime import datetime
 from typing import List, Dict, Tuple, Union, Optional
 
 from . import (
-    find_scan_axis,
     calculate_origin,
     create_attributes,
     set_dependency,
@@ -20,6 +19,7 @@ from . import (
 from .. import (
     imgcif2mcstas,
     split_arrays,
+    get_iso_timestamp,
     units_of_length,
     units_of_time,
     ureg,
@@ -57,10 +57,10 @@ def write_NXdata(
     nxsfile: h5py.File,
     datafiles: List[Path],
     goniometer: Dict,
-    data_type: str,
+    data_type: Tuple[str, int],
     coord_frame: str,
-    scan_range: Union[Tuple, np.ndarray],
-    scan_axis: str = None,
+    osc_scan: Dict[str, np.ndarray],
+    transl_scan: Dict[str, np.ndarray] = None,
 ):
     """
     Write NXdata group at entry/data
@@ -69,32 +69,26 @@ def write_NXdata(
         nxsfile (h5py.File):        NeXus file to be written.
         datafiles (List):           List of Path objects.
         goniometer (Dict):          Dictionary containing all the axes information.
-        data_type (str):            Images or events.
+        data_type (Tuple):          Images or events.
         coord_frame (str):          Coordinate system the axes are currently in.
-        scan_range (Tuple|array):   If writing events, this is just a (start, end) tuple.
-        scan_axis (str):            Rotation axis.
+        osc_scan (Dict):            Rotation scan. If writing events, this is just a (start, end) tuple.
+        transl_scan (Dict):         Scan along the xy (one or both) axes.
     """
     NXclass_logger.info("Start writing NXdata.")
     # Check that a valid datafile_list has been passed.
     assert len(datafiles) > 0, "Please pass at least a list of one HDF5 data file."
 
-    # If scan_axis hasn't been passed, identify it.
-    if not scan_axis:
-        scan_axis = find_scan_axis(
-            goniometer["axes"],
-            goniometer["starts"],
-            goniometer["ends"],
-            goniometer["types"],
-        )
+    # This assumes that a rotation scan is always passed
+    osc_axis, osc_range = list(osc_scan.items())[0]
 
     # Create NXdata group, unless it already exists, in which case just open it.
     nxdata = nxsfile.require_group("/entry/data")
     create_attributes(
         nxdata,
-        ("NX_class", "axes", "signal", scan_axis + "_indices"),
+        ("NX_class", "axes", "signal", osc_axis + "_indices"),
         (
             "NXdata",
-            scan_axis,
+            osc_axis,
             "data",
             [
                 0,
@@ -103,11 +97,11 @@ def write_NXdata(
     )
 
     # If mode is images, link to blank image data. Else go to events.
-    if data_type == "images":
+    if data_type[0] == "images":
         tmp_name = f"data_%0{6}d"
         for n, filename in enumerate(datafiles):
             nxdata[tmp_name % (n + 1)] = h5py.ExternalLink(filename.name, "data")
-    elif data_type == "events":
+    elif data_type[0] == "events":
         if len(datafiles) == 1 and "meta" in datafiles[0].as_posix():
             meta = datafiles
         else:
@@ -122,12 +116,13 @@ def write_NXdata(
         else:
             nxdata["meta_file"] = h5py.ExternalLink(meta[0].name, "/")
     else:
-        sys.exit("Please pass a correct data_type (images or events)")
+        raise SystemExit("Please pass a correct data_type (images or events)")
+        # sys.exit("Please pass a correct data_type (images or events)")
 
     # Write rotation axis dataset
-    ax = nxdata.create_dataset(scan_axis, data=scan_range)
-    idx = goniometer["axes"].index(scan_axis)
-    _dep = set_dependency(
+    ax = nxdata.create_dataset(osc_axis, data=osc_range)
+    idx = goniometer["axes"].index(osc_axis)
+    dep = set_dependency(
         goniometer["depends"][idx], path="/entry/sample/transformations/"
     )
 
@@ -137,22 +132,41 @@ def write_NXdata(
         ax,
         ("depends_on", "transformation_type", "units", "vector"),
         (
-            _dep,
+            dep,
             goniometer["types"][idx],
             goniometer["units"][idx],
-            vectors[scan_axis],
+            vectors[osc_axis],
         ),
     )
 
+    # If present, add linear/grid scan details
+    if transl_scan:
+        for k, v in transl_scan.items():
+            ax_dset = nxdata.create_dataset(k, data=v)
+            ax_idx = goniometer["axes"].index(k)
+            ax_dep = set_dependency(
+                goniometer["depends"][ax_idx], path="/entry/sample/transformations/"
+            )
+            create_attributes(
+                ax_dset,
+                ("depends_on", "transformation_type", "units", "vector"),
+                (
+                    ax_dep,
+                    goniometer["types"][ax_idx],
+                    goniometer["units"][ax_idx],
+                    vectors[k],
+                ),
+            )
+
 
 # NXsample
 def write_NXsample(
     nxsfile: h5py.File,
     goniometer: Dict,
     coord_frame: str,
-    data_type: str,
-    scan_axis: str,
-    scan_range: Union[Tuple, np.ndarray] = None,
+    data_type: Tuple[str, int],
+    osc_scan: Dict[str, np.ndarray],
+    transl_scan: Dict[str, np.ndarray] = None,
 ):
     """
     Write NXsample group at entry/sample
@@ -161,9 +175,9 @@ def write_NXsample(
         nxsfile (h5py.File):        NeXus file to be written
         goniometer (Dict):          Dictionary containing all the axes information
         coord_frame (str):          Coordinate system the axes are currently expressed in
-        data_type (str):            Images or events
-        scan_axis (str):            Rotation axis
-        scan_range (Tuple|array):   List/tuple/array of scan axis values
+        data_type (Tuple):            Images or events
+        osc_scan (Dict):            Rotation scan. If writing events, this is just a (start, end) tuple.
+        transl_scan (Dict):         Scan along the xy (one or both) axes.
     """
     NXclass_logger.info("Start writing NXsample and NXtransformations.")
     # Create NXsample group, unless it already exists, in which case just open it.
@@ -182,11 +196,20 @@ def write_NXsample(
         ("NXtransformations",),
     )
 
+    # Get rotation details
+    osc_axis, osc_range = list(osc_scan.items())[0]
+
     # Save sample depends_on
     nxsample.create_dataset(
-        "depends_on", data=set_dependency(scan_axis, path=nxtransformations.name)
+        "depends_on", data=set_dependency(osc_axis, path=nxtransformations.name)
     )
 
+    # Get xy details if passed
+    scan_axes = []
+    if transl_scan:
+        for k in transl_scan.keys():
+            scan_axes.append(k)
+
     # Create sample_{axisname} groups
     vectors = split_arrays(coord_frame, goniometer["axes"], goniometer["vectors"])
     for ax in goniometer["axes"]:
@@ -196,43 +219,62 @@ def write_NXsample(
             grp_name = "sample_" + ax
         nxsample_ax = nxsample.create_group(grp_name)
         create_attributes(nxsample_ax, ("NX_class",), ("NXpositioner",))
-        if ax == scan_axis:
+        if ax == osc_axis:
             # If we're dealing with the scan axis
-            idx = goniometer["axes"].index(scan_axis)
-            try:
-                for k in nxsfile["/entry/data"].keys():
-                    if isinstance(
-                        nxsfile["/entry/data"].get(k, getlink=True), h5py.ExternalLink
-                    ):
-                        # Don't even try to open!
-                        continue
-                    if nxsfile["/entry/data"][k].attrs.get("depends_on"):
-                        nxsample_ax[ax] = nxsfile[nxsfile["/entry/data"][k].name]
-                        nxtransformations[ax] = nxsfile[nxsfile["/entry/data"][k].name]
-            except KeyError:
-                nxax = nxsample_ax.create_dataset(ax, data=scan_range)
+            idx = goniometer["axes"].index(osc_axis)
+            # ... I was clearly overthinking this
+            if (
+                "data" in nxsfile["/entry"].keys()
+                and ax in nxsfile["/entry/data"].keys()
+            ):
+                nxsample_ax[ax] = nxsfile[nxsfile["/entry/data"][ax].name]
+                nxtransformations[ax] = nxsfile[nxsfile["/entry/data"][ax].name]
+            else:
+                nxax = nxsample_ax.create_dataset(ax, data=osc_range)
                 _dep = set_dependency(
                     goniometer["depends"][idx], path="/entry/sample/transformations/"
                 )
                 create_attributes(
-                    ax,
+                    nxax,
                     ("depends_on", "transformation_type", "units", "vector"),
                     (
                         _dep,
                         goniometer["types"][idx],
                         goniometer["units"][idx],
-                        vectors[scan_axis],
+                        vectors[osc_axis],
                     ),
                 )
                 nxtransformations[ax] = nxsfile[nxax.name]
-            # TODO handle the case where scan range has not been passed.
             # Write {axisname}_increment_set and {axis_name}_end datasets
-            if data_type == "images":
-                increment_set = np.repeat(
-                    goniometer["increments"][idx], len(scan_range)
-                )
+            if data_type[0] == "images":
+                increment_set = np.repeat(goniometer["increments"][idx], len(osc_range))
                 nxsample_ax.create_dataset(ax + "_increment_set", data=increment_set)
-                nxsample_ax.create_dataset(ax + "_end", data=scan_range + increment_set)
+                nxsample_ax.create_dataset(ax + "_end", data=osc_range + increment_set)
+        elif ax in scan_axes:
+            # For translations
+            if (
+                "data" in nxsfile["/entry"].keys()
+                and ax in nxsfile["/entry/data"].keys()
+            ):
+                nxsample_ax[ax] = nxsfile[nxsfile["/entry/data"][ax].name]
+                nxtransformations[ax] = nxsfile[nxsfile["/entry/data"][ax].name]
+            else:
+                idx = goniometer["axes"].index(ax)
+                nxax = nxsample_ax.create_dataset(ax, data=transl_scan[ax])
+                _dep = set_dependency(
+                    goniometer["depends"][idx], path="/entry/sample/transformations/"
+                )
+                create_attributes(
+                    nxax,
+                    ("depends_on", "transformation_type", "units", "vector"),
+                    (
+                        _dep,
+                        goniometer["types"][idx],
+                        goniometer["units"][idx],
+                        vectors[ax],
+                    ),
+                )
+                nxtransformations[ax] = nxsfile[nxax.name]
         else:
             # For all other axes
             idx = goniometer["axes"].index(ax)
@@ -252,6 +294,8 @@ def write_NXsample(
             )
             nxtransformations[ax] = nxsfile[nxax.name]
 
+    # Not the best but it works ...
+
     # Look for nxbeam in file, if it's there make link
     try:
         nxsample["beam"] = nxsfile["/entry/instrument/beam"]
@@ -369,18 +413,25 @@ def write_NXdetector(
             nxdetector[ll] = h5py.ExternalLink(meta.name, detector[ll])
     else:
         # Check for mask and flatfield files
+        # This is mostly for Tristan where there are external files
         # Flatfield
         if detector["flatfield"]:
             nxdetector.create_dataset(
                 "flatfield_applied", data=detector["flatfield_applied"]
             )
-            nxdetector.create_dataset("flatfield", data=detector["flatfield"])
+            flatfield = Path(detector["flatfield"])
+            image_key = "image" if "tristan" in detector["description"].lower() else "/"
+            nxdetector["flatfield"] = h5py.ExternalLink(flatfield.name, image_key)
+            # nxdetector.create_dataset("flatfield", data=detector["flatfield"])
         # Bad pixel mask
         if detector["pixel_mask"]:
             nxdetector.create_dataset(
                 "pixel_mask_applied", data=detector["pixel_mask_applied"]
             )
-            nxdetector.create_dataset("pixel_mask", data=detector["pixel_mask"])
+            mask = Path(detector["pixel_mask"])
+            image_key = "image" if "tristan" in detector["description"].lower() else "/"
+            nxdetector["pixel_mask"] = h5py.ExternalLink(mask.name, image_key)
+            # nxdetector.create_dataset("pixel_mask", data=detector["pixel_mask"])
 
     # Beam center
     # Check that the information hasn't already been written by the meta file.
@@ -669,6 +720,36 @@ def write_NXcollection(
         )
 
 
+# NXdatetime writer
+def write_NXdatetime(nxsfile: h5py.File, timestamps: Tuple):
+    """
+    Write start and end timestamps under /entry/start_time and /entry/end_time.
+
+    Args:
+        nxsfile (h5py.File): Nexus file to be written.
+        timestamps (Tuple): Timestamps (start, end).
+    """
+    nxentry = nxsfile.require_group("entry")
+
+    start = timestamps[0]
+    if start:
+        if type(start) is datetime:
+            start = start.strftime("%Y-%m-%dT%H:%M:%S")
+            start = get_iso_timestamp(start)
+        if start.endswith("Z") is False:  # Just in case
+            start += "Z"
+        nxentry.create_dataset("start_time", data=np.string_(start))
+
+    stop = timestamps[1]
+    if stop:
+        if type(stop) is datetime:
+            stop = stop.strftime("%Y-%m-%dT%H:%M:%S")
+            stop = get_iso_timestamp(stop)
+        if stop.endswith("Z") is False:
+            stop += "Z"
+        nxentry.create_dataset("end_time", data=np.string_(stop))
+
+
 # NXnote writer
 # To be used e.g. as a place to store pump-probe info such as pump delay/width
 def write_NXnote(nxsfile: h5py.File, loc: str, info: Dict):
