@@ -10,7 +10,13 @@ import numpy as np
 from pathlib import Path
 from typing import Dict, List, Tuple, Union
 
-from . import find_scan_axis, calculate_scan_range, find_number_of_images
+from . import (
+    find_osc_axis,
+    calculate_rotation_scan_range,
+    find_grid_scan_axes,
+    calculate_grid_scan_range,
+    find_number_of_images,
+)
 from .. import units_of_time
 
 from .NXclassWriters import (
@@ -21,6 +27,7 @@ from .NXclassWriters import (
     write_NXsource,
     write_NXdetector,
     write_NXdetector_module,
+    write_NXdatetime,
 )
 
 from ..tools.MetaReader import overwrite_beam, overwrite_detector
@@ -76,15 +83,18 @@ def write_nexus(
         link_list = None
     writer_logger.info("Writing NXmx NeXus file ...")
 
-    # Identify scan axis
-    osc_axis = find_scan_axis(
+    # Define a SCANS dictionary to save both rotation and eventually translation scan ranges
+    SCANS = {}
+
+    # Identify rotation scan axis
+    osc_axis = find_osc_axis(
         goniometer.axes, goniometer.starts, goniometer.ends, goniometer.types
     )
     idx = goniometer.axes.index(osc_axis)
 
     if detector.mode == "events":
         data_type = ("events", len(datafiles))
-        scan_range = (goniometer.starts[idx], goniometer.ends[idx])
+        osc_range = (goniometer.starts[idx], goniometer.ends[idx])
     else:
         # Find total number of images that have been written across the files.
         if len(datafiles) == 1:
@@ -95,30 +105,62 @@ def write_nexus(
         data_type = ("images", num_images)
         writer_logger.info(f"Total number of images: {num_images}")
 
-        # Compute scan_range
+        # Compute rotation scan_range
         if goniometer.increments[idx] != 0.0:
-            scan_range = calculate_scan_range(
+            osc_range = calculate_rotation_scan_range(
                 goniometer.starts[idx],
                 goniometer.ends[idx],
                 axis_increment=goniometer.increments[idx],
             )
         else:
-            scan_range = calculate_scan_range(
+            osc_range = calculate_rotation_scan_range(
                 goniometer.starts[idx], goniometer.ends[idx], n_images=num_images
             )
 
-    writer_logger.info(f"Scan axis: {osc_axis}")
-    writer_logger.info(f"Scan from {scan_range[0]} tp {scan_range[-1]}.")
+    SCANS["rotation"] = {osc_axis: osc_range}
 
-    nxentry = write_NXentry(nxsfile)
+    writer_logger.info(f"Rotatin scan axis: {osc_axis}")
+    writer_logger.info(f"Scan from {osc_range[0]} tp {osc_range[-1]}.")
+
+    # Look for a translation scan (usually on xy)
+    transl_axes = find_grid_scan_axes(
+        goniometer.axes, goniometer.starts, goniometer.ends, goniometer.types
+    )
+    # If xy scan axes are identified, add to dictionary
+    if len(transl_axes) > 0:
+        writer_logger.info(f"Scan along {transl_axes} axes.")
+        tr_idx = [goniometer.axes.index(j) for j in transl_axes]
+        transl_starts = [goniometer.starts[i] for i in tr_idx]
+        transl_ends = [goniometer.ends[i] for i in tr_idx]
+        transl_increments = [goniometer.increments[i] for i in tr_idx]
+        for tr in range(len(transl_axes)):
+            writer_logger.info(
+                f"{transl_axes[tr]} scan from {transl_starts[tr]} to {transl_ends[tr]}, with a step of {transl_increments[tr]}"
+            )
+        # TODO decide what to do for n_images=(nx,ny) in this case...
+        # Tbh, it should work without it anyway
+        transl_range = calculate_grid_scan_range(
+            transl_axes,
+            transl_starts,
+            transl_ends,
+            transl_increments,
+        )  # NB. leaving snaked = False for demo. TODO change at some point.
+        SCANS["translation"] = transl_range
+
+        # Just a check
+        ax1 = transl_axes[0]
+        assert num_images == len(
+            transl_range[ax1]
+        ), "The total number of images doesn't match the number of scan points, please double check the input."
+
+    write_NXentry(nxsfile)
 
     # Call the writers
     call_writers(
         nxsfile,
         datafiles,
         coordinate_frame,
-        osc_axis,
-        scan_range,
+        SCANS,
         data_type,
         goniometer.__dict__,
         detector.__dict__,
@@ -144,9 +186,11 @@ def write_nexus(
 
     # NX_DATE_TIME: /entry/start_time and /entry/end_time
     if timestamps[0] is not None:
-        nxentry.create_dataset("start_time", data=np.string_(timestamps[0]))
+        write_NXdatetime(nxsfile, (timestamps[0], None))
+        # nxentry.create_dataset("start_time", data=np.string_(timestamps[0]))
     if timestamps[1] is not None:
-        nxentry.create_dataset("end_time", data=np.string_(timestamps[1]))
+        write_NXdatetime(nxsfile, (None, timestamps[1]))
+        # nxentry.create_dataset("end_time", data=np.string_(timestamps[1]))
 
 
 def write_nexus_demo(
@@ -184,30 +228,71 @@ def write_nexus_demo(
     """
     writer_logger.info("Writing NXmx demo ...")
     writer_logger.info(f"The data file will contain {data_type[1]} {data_type[0]}")
-    # Identify scan axis
-    osc_axis = find_scan_axis(
+    # Define a SCANS dictionary to save both rotation and eventually translation scan ranges
+    SCANS = {}
+
+    # Identify rotation scan axis
+    osc_axis = find_osc_axis(
+        goniometer.axes, goniometer.starts, goniometer.ends, goniometer.types
+    )
+    # Look for a translation scan (usually on xy)
+    transl_axes = find_grid_scan_axes(
         goniometer.axes, goniometer.starts, goniometer.ends, goniometer.types
     )
 
-    # Compute scan_range
+    # NB. doing this first so that if number of images is None, it can be overwritten.
+    # If xy scan axes are identified, add to dictionary
+    if len(transl_axes) > 0:
+        writer_logger.info(f"Scan along {transl_axes} axes.")
+        tr_idx = [goniometer.axes.index(j) for j in transl_axes]
+        transl_starts = [goniometer.starts[i] for i in tr_idx]
+        transl_ends = [goniometer.ends[i] for i in tr_idx]
+        transl_increments = [goniometer.increments[i] for i in tr_idx]
+        for tr in range(len(transl_axes)):
+            writer_logger.info(
+                f"{transl_axes[tr]} scan from {transl_starts[tr]} to {transl_ends[tr]}, with a step of {transl_increments[tr]}"
+            )
+        transl_range = calculate_grid_scan_range(
+            transl_axes,
+            transl_starts,
+            transl_ends,
+            transl_increments,
+        )  # NB. leaving snaked = False for demo. TODO change at some point.
+        SCANS["translation"] = transl_range
+
+    # TODO FIXME the number of images should come from CLI if it's a xy scan.
+    # Compute scan_range for rotation axis
     idx = goniometer.axes.index(osc_axis)
     if data_type[0] == "images":
-        if data_type[1] is None:
-            scan_range = calculate_scan_range(
+        if data_type[1] is None and len(transl_axes) == 0:
+            osc_range = calculate_rotation_scan_range(
                 goniometer.starts[idx],
                 goniometer.ends[idx],
                 axis_increment=goniometer.increments[idx],
             )
-            data_type = ("images", len(scan_range))
+            data_type = ("images", len(osc_range))
+        elif data_type[1] is None and len(transl_axes) > 0:
+            ax1 = transl_axes[0]
+            num_imgs = len(transl_range[ax1])
+            osc_range = calculate_rotation_scan_range(
+                goniometer.starts[idx], goniometer.ends[idx], n_images=num_imgs
+            )
+            data_type = ("images", num_imgs)
         else:
-            scan_range = calculate_scan_range(
+            ax1 = transl_axes[0]
+            assert data_type[1] == len(
+                transl_range[ax1]
+            ), "The total number of images doesn't match the number of scan points, please double check the input."
+            osc_range = calculate_rotation_scan_range(
                 goniometer.starts[idx], goniometer.ends[idx], n_images=data_type[1]
             )
     elif data_type[0] == "events":
-        scan_range = (goniometer.starts[idx], goniometer.ends[idx])
+        osc_range = (goniometer.starts[idx], goniometer.ends[idx])
 
-    writer_logger.info(f"Scan axis: {osc_axis}.")
-    writer_logger.info(f"Scan from {scan_range[0]} to {scan_range[-1]}.")
+    SCANS["rotation"] = {osc_axis: osc_range}
+
+    writer_logger.info(f"Rotation scan axis: {osc_axis}.")
+    writer_logger.info(f"Scan from {osc_range[0]} to {osc_range[-1]}.")
 
     # Figure out how many files will need to be written
     writer_logger.info("Calculating number of files to write ...")
@@ -248,8 +333,7 @@ def write_nexus_demo(
         nxsfile,
         datafiles,
         coordinate_frame,
-        osc_axis,
-        scan_range,
+        SCANS,
         data_type,
         goniometer.__dict__,
         detector.__dict__,
@@ -277,10 +361,9 @@ def write_nexus_demo(
 # def call_writers(*args,**kwargs):
 def call_writers(
     nxsfile: h5py.File,
-    datafiles: List[Path],
+    datafiles: List[Union[Path, str]],
     coordinate_frame: str,
-    scan_axis: str,
-    scan_range: Union[Tuple, np.ndarray],
+    SCANS: Dict,
     data_type: Tuple[str, int],
     goniometer: Dict,
     detector: Dict,
@@ -288,22 +371,37 @@ def call_writers(
     source: Dict,
     beam: Dict,
     attenuator: Dict,
-    metafile: Path = None,
+    metafile: Union[Path, str] = None,
     link_list: List = None,
 ):
     """ Call the writers for the NeXus base classes."""
     logger = logging.getLogger("NeXusGenerator.writer.call")
     logger.info("Calling the writers ...")
 
+    # Get scan details first
+    osc_scan = SCANS["rotation"]
+    if "translation" in SCANS.keys():
+        transl_scan = SCANS["translation"]
+    else:
+        transl_scan = None
+
+    # Check that filenames are paths
+    if all(isinstance(f, Path) for f in datafiles) is False:
+        datafiles = [Path(f).expanduser().resolve() for f in datafiles]
+
+    if metafile:
+        if type(metafile) is str:
+            metafile = Path(metafile).expanduser().resolve()
+
     # NXdata: entry/data
     write_NXdata(
         nxsfile,
         datafiles,
         goniometer,
-        data_type[0],
+        data_type,
         coordinate_frame,
-        scan_range,
-        scan_axis,
+        osc_scan,
+        transl_scan,
     )
 
     # NXinstrument: entry/instrument
@@ -342,7 +440,7 @@ def call_writers(
         nxsfile,
         goniometer,
         coordinate_frame,
-        data_type[0],
-        scan_axis,
-        scan_range,
+        data_type,
+        osc_scan,
+        transl_scan,
     )
