@@ -1,14 +1,17 @@
 """
 Utilities for writing new NeXus format files.
 """
-import sys
+
 import math
 import h5py
 import numpy as np
 
 from pathlib import Path
 from h5py import AttributeManager
-from typing import List, Tuple, Union
+from typing import List, Dict, Tuple, Union
+
+from scanspec.core import Path as ScanPath
+from scanspec.specs import Line
 
 
 def create_attributes(
@@ -50,7 +53,7 @@ def set_dependency(dep_info: str, path: str = None):
         return np.string_(dep_info)
 
 
-def find_scan_axis(
+def find_osc_axis(
     axes_names: List,
     axes_starts: List,
     axes_ends: List,
@@ -72,7 +75,7 @@ def find_scan_axis(
         axes_types (list):      List of axes types, useful to identify only the rotation axes.
         default (str):          String to deafult to in case scan axis is not found.
     Returns:
-        scan_axis (str):        String identifying the scan axis.
+        scan_axis (str):        String identifying the rotation scan axis.
     """
     # This assumes that at least one rotation axis is always passed.
     # Assuming all list are of the same length ...
@@ -93,11 +96,12 @@ def find_scan_axis(
         elif idx.count(True) == 1:
             scan_axis = axes_names[idx.index(True)]
         else:
-            sys.exit("Unable to correctly identify the scan axis.")
+            raise SystemExit("Unable to correctly identify the scan axis.")
+            # sys.exit("Unable to correctly identify the scan axis.")
     return scan_axis
 
 
-def calculate_scan_range(
+def calculate_rotation_scan_range(
     axis_start: float,
     axis_end: float,
     axis_increment: float = None,
@@ -106,7 +110,7 @@ def calculate_scan_range(
     """
     Calculate the scan range for a rotation collection and return as a numpy array.
     For this calculation axes_increments and n_images are mutually exclusive.
-    If there are multiple images but no rotation scan, renurn a numpy array of axis_start repeated n_images times.
+    If there are multiple images but no rotation scan, return a numpy array of axis_start repeated n_images times.
 
     Args:
         axis_start (float):         Rotation axis position at the beginning of the scan, float.
@@ -126,6 +130,93 @@ def calculate_scan_range(
     return scan_range
 
 
+def find_grid_scan_axes(
+    axes_names: List,
+    axes_starts: List,
+    axes_ends: List,
+    axes_types: List,
+) -> List[str]:
+    """
+    Identify the scan axes for a linear/grid scan.
+
+    Args:
+        axes_names (List): List of names associated to goniometer axes.
+        axes_starts (List): List of start values.
+        axes_ends (List): List of end values.
+        axes_types (List): List of axes types, useful to identify only the translation axes.
+
+    Returns:
+        List[str]: List of strings identifying the linear/grid scan axes. If no axes are identified, it will return an empty list.
+    """
+    assert len(axes_names) > 0, "Please pass at least one axis"
+
+    # Look only at translation axes
+    grid_idx = [i for i in range(len(axes_names)) if axes_types[i] == "translation"]
+    axes_names = [axes_names[j] for j in grid_idx]
+    axes_starts = [axes_starts[j] for j in grid_idx]
+    axes_ends = [axes_ends[j] for j in grid_idx]
+
+    scan_axis = []
+    for n, ax in enumerate(axes_names):
+        if axes_starts[n] != axes_ends[n]:
+            scan_axis.append(ax)
+    return scan_axis
+
+
+def calculate_grid_scan_range(
+    axes_names: List,
+    axes_starts: List,
+    axes_ends: List,
+    axes_increments: List = None,
+    n_images: Union[Tuple, int] = None,
+    snaked: bool = False,
+) -> Dict[str, np.ndarray]:
+    """
+    Calculate the scan range for a linear/grid scan from the number of images to be written.
+    If the number of images is not provided, it can be calculated from the increment value of the axis.
+
+    Args:
+        axes_names (List): List of names for the axes involved in the scan.
+        axes_starts (List): List of axis positions at the beginning of the scan.
+        axes_ends (List): List of axis positions at the end of the scan.
+        axes_increments (List, optional): List of ranges through which the axes move each frame. Defaults to None.
+        n_images (Tuple|int, optional): Number of images to be written. If writing a 2D scan, it should be a (nx, ny) tuple, \
+                                        where tot_n_img=nx*ny. Defaults to None.
+        snaked (bool): If True, scanspec will "draw" a snaked grid. Defaults to False.
+
+    Returns:
+        Dict[str, np.ndarray]: A dictionary of ("axis_name": axis_range) key-value pairs.
+    """
+    # Just to double check that nothing weird is going on
+    # assert len(axes_names) > 2, "Too many axes for a linear or grid scan."
+
+    if len(axes_names) == 1:
+        if not n_images:
+            n_images = int(abs(axes_starts[0] - axes_ends[0]) / axes_increments[0])
+        spec = Line(axes_names[0], axes_starts[0], axes_ends[0], n_images)
+        scan_path = ScanPath(spec.calculate())
+    else:
+        if not n_images:
+            n_images0 = int(abs(axes_starts[0] - axes_ends[0]) / axes_increments[0])
+            n_images1 = int(abs(axes_starts[1] - axes_ends[1]) / axes_increments[1])
+        else:
+            # FIXME Need to be careful with n_images, it's not the total that should be passed here.
+            # Tot number of images (those passed in CLI or from beamline I guess) = n0*n1
+            n_images0 = n_images[0]
+            n_images1 = n_images[1]
+        if snaked is True:
+            spec = Line(axes_names[0], axes_starts[0], axes_ends[0], n_images0) * ~Line(
+                axes_names[1], axes_starts[1], axes_ends[1], n_images1
+            )
+        else:
+            spec = Line(axes_names[0], axes_starts[0], axes_ends[0], n_images0) * Line(
+                axes_names[1], axes_starts[1], axes_ends[1], n_images1
+            )
+        scan_path = ScanPath(spec.calculate())
+
+    return scan_path.consume().midpoints
+
+
 def calculate_origin(
     beam_center_fs: Union[List, Tuple],
     fs_pixel_size: Union[List, Tuple],
